package com.fr.plugin.decision.third.sms.fun;

import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class SMSXmlReader {

    private static final Map<String, Map<String, Object>> SMS_TEMPLATES_MAP = new ConcurrentHashMap<>();
    private static final String SMS_PUBLIC_MODEL = "public-model";
    private static final String SMS_PRIVATE_MODEL = "private-model";

    private static final String SMS_TEMPLATE_XML = "sms_template.xml";
    private static final String SMS_TEMPLATE_MAPPING_XML = "sms_template_mapping.xml";

    public static Map<String, Object> getPublicSMSTemplates() {
        return getSMSTemplates(SMS_TEMPLATE_XML, SMS_PUBLIC_MODEL);
    }

    public static Map<String, Object> getPrivateSMSTemplates() {
        return getSMSTemplates(SMS_TEMPLATE_XML, SMS_PRIVATE_MODEL);
    }

    public static Map<String, Object> getSMSTemplateMappings() {
        return getSMSTemplates(SMS_TEMPLATE_MAPPING_XML, SMS_PUBLIC_MODEL);
    }

    public static Map<String, Object> getSMSTemplates(String xmlPath, String type) {
        if (!SMS_TEMPLATES_MAP.containsKey(type)) {
            synchronized (com.fr.base.sms.SMSXmlReader.class) {
                Map<String, Object> map = new ConcurrentHashMap<>();
                readXML(map, xmlPath, type);
                if (!map.isEmpty()) {
                    SMS_TEMPLATES_MAP.put(type, map);
                }
            }
        }
        return SMS_TEMPLATES_MAP.get(type);
    }

    private static void readXML(Map<String, Object> map, String xmlFile, String nodeName) {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        String xmlPath = "com/fr/plugin/decision/third/sms/xml/".concat(xmlFile);
        try {
            dbf.setFeature("http://apache.org/xml/features/disallow-doctype-decl", true);
            dbf.setFeature("http://xml.org/sax/features/external-general-entities", false);
            dbf.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
            dbf.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
            dbf.setXIncludeAware(false);
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document document = db.parse(SMSXmlReader.class.getClassLoader().getResourceAsStream(xmlPath));
            NodeList bookList = document.getElementsByTagName(nodeName);
            for (int i = 0; i < bookList.getLength(); i++) {
                Node node = bookList.item(i);
                String id = node.getAttributes().getNamedItem("id").getTextContent();
                JSONObject jsonObject = JSONObject.create();
                jsonObject.put("id", id);
                jsonObject.put("content", node.getAttributes().getNamedItem("content").getTextContent());
                jsonObject.put("language", node.getAttributes().getNamedItem("language").getTextContent());
                map.put(id, jsonObject);
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }
}
