package com.fr.plugin.decision.third.sms.fun;

import com.fr.base.sms.SMSTemplateType;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.stable.StringUtils;
import com.fr.stable.fun.SMSServiceProvider;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author richie
 * @version 10.0
 * Created by richie on 2019/11/8
 * 这个是短信发送的客户端，调用各家实际短信发送平台，这里只通过输出日志的方式提供一个示例
 */
public class SmsClient {

    private static final SmsClient instance = new SmsClient();
    private static final String SMS_PUBLIC_MODEL = "public-model";
    private static final String SMS_PRIVATE_MODEL = "private-model";

    public static SmsClient getInstance() {
        return instance;
    }

    public String getSign() {
        return "第三方短信";
    }

    /**
     * 获取短信配置信息
     *
     * @return 短信配置信息
     */
    public Map<String, Object> getSmsConfig() {
        Map<String, Object> result = new HashMap<>();
        result.put("username", "优先级20");
        result.put("balance", 102.20);
        result.put("isSmsAvailable", true);
        result.put("accountType", 1);
        result.put("publicModel", JSONArray.create(SMSXmlReader.getPublicSMSTemplates()));
        result.put("privateModel", JSONArray.create(SMSXmlReader.getPrivateSMSTemplates()));
        result.put("sign", getSign());

        initSmsModels(result, "publicModel");
        initSmsModels(result, "privateModel");
        return result;
    }

    public boolean send(String mobile) {
        System.out.println("新插件（20）发送测试短信成功");
        FineLoggerFactory.getLogger().info("新插件（20）发送测试短信成功");
        return true;
    }

    public boolean send(String template, String mobile, JSONObject para, String receiver) {
        System.out.println("新插件（20）发送普通短信成功");
        FineLoggerFactory.getLogger().info("新插件（20）发送普通短信成功");
        return true;
    }

    public boolean batchSend(String template, List<String> mobiles, JSONArray params, List<String> receivers) {
        System.out.println("新插件（20）发送批量短信成功");
        FineLoggerFactory.getLogger().info("新插件（20）发送批量短信成功");
        return true;
    }

    private void initSmsModels(Map<String, Object> result, String key) {
        if (result.containsKey(key)) {
            List<Map<String, Object>> smsModels = new ArrayList<Map<String, Object>>();
            JSONArray ja = (JSONArray) result.get(key);
            for (int i = 0; i < ja.length(); i++) {
                JSONObject jo = ja.optJSONObject(i);
                Map<String, Object> map = new HashMap<String, Object>();
                String language = jo.optString("language");
                if (StringUtils.isNotEmpty(language) && StringUtils.equals(key, "publicModel") && !StringUtils.equals(language, SMSTemplateType.MAINLAND_SMS_TEMPLATE.getLanguage())) {
                    continue;
                }
                map.put("id", jo.optString("id"));
                map.put("content", jo.optString("content"));
                map.put("language", language);
                smsModels.add(map);
            }
            result.put(key, smsModels);
        }
    }

    public Map<String, Map<String, Object>> getSMSTemplate() {
        Map<String, Map<String, Object>> result = new HashMap<>();
        result.put(SMS_PUBLIC_MODEL, SMSXmlReader.getPublicSMSTemplates());
        result.put(SMS_PRIVATE_MODEL, SMSXmlReader.getPrivateSMSTemplates());
        return result;
    }

}