// 邮箱配置组件的注册过程一共分为4步：

// 1.基础配置
var PRIORITY = 20; // 优先级（需要跟ThirdSmsServiceProvider#priority方法返回值一致），必须大于1，当有多个组件注册时，只有优先级最高的组件才会起作用
var WIDGET_TYPE = 'demo.system.sms.config'; // 组件名称
var MODEL_TYPE = 'demo.model.system.sms.config'; // 模型名称

// 2.定义组件
var LAYOUT = {
  ITEM_HEIGHT: 24,
  ITEM_GAP: 15,
  LABEL_WIDTH: 120,
};
var Widget = BI.inherit(BI.Pane, {
  props: {
    baseCls: 'demo-system-sms-config',
  },

  _store: function () {
    return BI.Models.getModel(MODEL_TYPE);
  },

  watch: {
    'config.balance': function (value) {
      this.balanceLabel.setText(config.balance + BI.i18nText('Dec-Basic_RMB'));
      this.testButton.setEnable(value > 0);
    },
  },

  beforeRender: function (callback) {
    var self = this;
    self.loading();
    this.store.requestGetConfig(function () {
      callback();
      self.loaded();
    });
  },

  render: function () {
    var self = this;
    var config = self.model.config;

    return {
      type: 'bi.vertical',
      bgap: LAYOUT.ITEM_GAP,
      items: [
        // 账号绑定
        {
          type: 'bi.vertical_adapt',
          height: LAYOUT.ITEM_HEIGHT,
          items: [
            {
              type: 'bi.label',
              cls: 'dec-font-weight-bold',
              width: LAYOUT.LABEL_WIDTH,
              text: BI.i18nText('Dec-System_SMS_Account_Bind'),
              textAlign: 'left',
              title: BI.i18nText('Dec-System_SMS_Account_Bind'),
            },
            {
              type: 'bi.label',
              text: config.username,
              textAlign: 'left',
            },
          ],
        },
        // 账号余额
        {
          type: 'bi.vertical_adapt',
          height: LAYOUT.ITEM_HEIGHT,
          items: [
            {
              type: 'bi.label',
              cls: 'dec-font-weight-bold',
              width: LAYOUT.LABEL_WIDTH,
              text: BI.i18nText('Dec-System_SMS_Account_Balance'),
              textAlign: 'left',
              title: BI.i18nText('Dec-System_SMS_Account_Balance'),
            },
            {
              type: 'bi.label',
              ref: function (_ref) {
                self.balanceLabel = _ref;
              },
              text: config.balance + BI.i18nText('Dec-Basic_RMB'),
              textAlign: 'left',
            },
          ],
        },
        // 短信签名
        {
          type: 'bi.vertical_adapt',
          height: LAYOUT.ITEM_HEIGHT,
          items: [
            {
              type: 'bi.label',
              cls: 'dec-font-weight-bold',
              width: LAYOUT.LABEL_WIDTH,
              text: BI.i18nText('Dec-System_SMS_Signatures'),
              textAlign: 'left',
              title: BI.i18nText('Dec-System_SMS_Signatures'),
            },
            {
              type: 'bi.label',
              text: config.sign,
              textAlign: 'left',
            },
          ],
        },
        // 信息测试
        {
          type: 'bi.vertical_adapt',
          height: LAYOUT.ITEM_HEIGHT,
          items: [
            {
              type: 'bi.label',
              cls: 'dec-font-weight-bold',
              width: LAYOUT.LABEL_WIDTH,
              text: BI.i18nText('Dec-System_SMS_Test'),
              textAlign: 'left',
              title: BI.i18nText('Dec-System_SMS_Test'),
            },
            {
              type: 'bi.button',
              ref: function (_ref) {
                self.testButton = _ref;
              },
              text: BI.i18nText('Dec-System_Send_Test_SMS'),
              textAlign: 'left',
              clear: true,
              disabled: config.balance <= 0,
              handler: function () {
                self._sendTestMessage();
              },
            },
          ],
        },
      ],
    };
  },

  /**
   * 发送测试短信
   */
  _sendTestMessage: function () {
    var self = this;
    var popoverName = BI.UUID();
    BI.Popovers.create(popoverName, {
      type: 'bi.popover',
      width: 450,
      height: 220,
      header: BI.i18nText('Dec-System_Send_Test_Sms'),
      body: {
        type: 'dec.send.test.msg.dialog',
        listeners: [
          {
            eventName: BI.Popover.EVENT_CONFIRM,
            action: function (phone) {
              self.store.requsetSendTestMessage(phone);
            },
          },
          {
            eventName: BI.Popover.EVENT_CLOSE,
            action: function () {
              BI.Popovers.remove(popoverName);
            },
          },
        ],
      },
    });
    BI.Popovers.show(popoverName);
  },
});

// 3.定义模型
var Model = BI.inherit(Fix.Model, {
  state: function () {
    return {
      config: {
        isSmsAvailable: true,
        accountType: 1,
        username: '',
        balance: 0,
        sign: '',
        publicModel: [],
        privateModel: [],
      },
    };
  },

  actions: {
    /**
     * 获取邮箱配置
     * @param {Function} callback beforeRender的回调
     */
    requestGetConfig: function (callback) {
      var self = this;
      // 发送请求，获取配置
      Dec.reqGetHandle('/v10/third/sms', '', function (data) {
        self.model.config = data;
        BI.isFunction(callback) && callback();
      });
    },

    /**
     * 发送测试邮件
     * @param {string} phone 收件人号码
     */
    requsetSendTestMessage: function (phone) {
      var self = this;
      Dec.reqPost(
        '/v10/config/sms/test',
        {
          mobile: phone,
        },
        function (result) {
          if (result && result.data && result.data.code === 'success') {
            BI.Msg.toast(BI.i18nText('Dec-Send_Success'), { level: 'success' });
          } else {
            BI.Msg.toast(BI.i18nText('Dec-System_SMS_Send_Fail'), {
              level: 'error',
            });
          }
          self.requestGetConfig();
        }
      );
    },
  },
});

// 4.注册组件
BI.shortcut(WIDGET_TYPE, Widget); // 注册组件
BI.model(MODEL_TYPE, Model); // 注册模型
BI.config('dec.provider.system', function (provider) {
  provider.registerSmsConfigWidget(WIDGET_TYPE, PRIORITY); // 注册邮箱配置组件
});
